////
// Calculate the massless limit at LO and NLO
///

#include "Massless_lim.H"
#include "Massless_Kernels.H"

#include <iostream>
#include <vector>
#include <cmath>

using namespace std;

double Massless_Limit::dsigmagg_lo(double y, double z)
{
  double ds(0.), m2b(p_lumi->m2bext()), L(log(m2h/m2b)/(2.*M_PI));
  double asr(2.*M_PI*p_lumi->ass(m2r));
  ds = asr*asr*s0*(Agg_2_2(y,z)*L*L + Agg_2_1(y,z)*L + Agg_2_0(y)); // Eq (29) of FONLL-A paper
  return ds;                                                        // A20 is not convoluted with anything, z variable of convolution
}

double Massless_Limit::dsigmaqqb_lo(double y, double z) // This term should be the same as the one in the 5F but at the moment it doe not work
{
  double ds(0.);
  double asr(2.*M_PI*p_lumi->ass(m2r));
  double mylfh=0., mylfr=0.;
  ds = asr*asr*s0*deltaqqba_(&y,&mylfh,&mylfr)/M_PI/M_PI/(1. - y); // Tutte le cose costanti in z sono divise per il Jacobiano
  return ds;
}

double Massless_Limit::Agg_2_0(double y) // This term should be the same as the one in the 5F but at the moment it doe not work
{
  // constant in z!
  double mylfh=0., mylfr=0.;
  return deltagga_(&y,&mylfh,&mylfr)/M_PI/M_PI/(1.-y); // deltagga e' una funzione NNLO di Harlander
}

double Massless_Limit::Agg_2_1(double y, double z)
{
  double pqg = p_lumi->Pqg(z);
  double yz  = y/z;
  double mylfh=0., mylfr=0.;
  return 4.*pqg*sbg1_(&yz,&mylfh)/M_PI;
}

double Massless_Limit::Agg_2_2(double y, double z)
{
  double pqg_1 = p_lumi->Pqg(z);
  double pqg_2 = p_lumi->Pqg(y/z);
  return 2.*y*pqg_1*pqg_2/z; // 2 is b=b~
}

double Massless_Limit::L_qq(const double x, const double y)
{
  double lqq(0.);
  lqq +=  p_lumi->Lumi(4,6,x,y,1.,1.,10);
  lqq +=  p_lumi->Lumi(6,4,x,y,1.,1.,10);
  lqq +=  p_lumi->Lumi(3,7,x,y,1.,1.,10);
  lqq +=  p_lumi->Lumi(7,3,x,y,1.,1.,10);
  lqq +=  p_lumi->Lumi(2,8,x,y,1.,1.,10);
  lqq +=  p_lumi->Lumi(8,2,x,y,1.,1.,10);
  lqq +=  p_lumi->Lumi(1,9,x,y,1.,1.,10);
  lqq +=  p_lumi->Lumi(9,1,x,y,1.,1.,10);
  return lqq;
  
}

double Massless_Limit::L_gq(const double x, const double y)
{
  double lgq(0.);
  lgq +=  p_lumi->Lumi(5,6,x,y,1.,1.,10);
  lgq +=  p_lumi->Lumi(5,4,x,y,1.,1.,10);
  lgq +=  p_lumi->Lumi(5,7,x,y,1.,1.,10);
  lgq +=  p_lumi->Lumi(5,3,x,y,1.,1.,10);
  lgq +=  p_lumi->Lumi(5,8,x,y,1.,1.,10);
  lgq +=  p_lumi->Lumi(5,2,x,y,1.,1.,10);
  lgq +=  p_lumi->Lumi(5,9,x,y,1.,1.,10);
  lgq +=  p_lumi->Lumi(5,1,x,y,1.,1.,10);
  return lgq;
  
}

double Massless_Limit::dsigma0LO(double z[], size_t dim, void* p)
{
  double dsg(0.), x,y;
  double tau, jac(1.), xi;
  double t, tmin;

  double lgg,lqq;
  double sgg(0.), sqq(0.);

  tau  = tauh + (1. - tauh)*z[0];
  jac *= (1. - tauh);
  double ximax = - 0.5*log(tau);
  xi   = -ximax + 2.*ximax*z[1];
  jac *=  2.*ximax;

  //standard variables  x, y ;
  x = sqrt(tau)*exp(xi);
  y = sqrt(tau)*exp(-xi);
  
  tmin = y;
  t    = tmin + (1. - tmin)*z[2];
  jac *= (1. - tmin);

  // gg
  sgg        =       dsigmagg_lo(y,t);
  // q q~
  sqq        =       dsigmaqqb_lo(y,t);
  
  lgg     = p_lumi->Lumi(5,5,x,tauh/tau,1.,1.,10);
  lqq     = L_qq(x,tauh/tau);

  dsg = jac*( lqq * sqq
              + lgg * sgg )/tau;
  return dsg;
}



