#include "Massless_lim.H"
#include "Massless_Kernels.H"
#include "PDF.H"

#include <cmath>
#include <iostream>


using namespace std;


double Massless_Limit::dsigma0NLO(double z[], size_t dim, void *p)
{
  double res(0.);
  // first round of variables
  double tau,xi,ximax,jac(1.),x,y;
  // lumis and MEs
  double lgg,lqg,lgq,lqq,
    sgg,sgq,sqg, sqq;
  //second variables
  double zz, tt;
  double asr(2.*M_PI*Massless_Limit::p_lumi->ass(m2r));

  double acut = m_amin;//0.7e-2;
  double MAX  = 1. - acut;

  if(z[0] < MAX && z[1] < MAX && z[2] < MAX && z[3] < MAX){
    //  tau  = tauh + (1. - tauh)*z[0];
    //  jac *= (1. - tauh);
    tau = tauh*pow(1/tauh,z[0]);
    jac *= - log(tauh);

    ximax = -1./2.*log(tau);
    xi    = -ximax + 2.*ximax*z[1];
    jac  *= 2.*ximax;

    x = sqrt(tau)*exp(xi);
    y = sqrt(tau)*exp(-xi);

    lgg = Massless_Limit::p_lumi->Lumi(5,5,x,tauh/tau,1.,1.,10);
    lgq = Massless_Limit::L_gq(x, tauh/tau);
    lqg = Massless_Limit::L_gq(tauh/tau, x);
    lqq = Massless_Limit::L_qq(x,tauh/tau);

    //  zz = y + (1. - y)*z[2];
    //  jac *= (1. - y);
    zz  = y*pow(1/y,z[2]);
    jac *= - log(y);

    tt = y/zz + (1. - y/zz)*z[3];
    jac *= (1. - y/zz);

    sgg  = dsigmagg_nlo(y,zz,tt);
    
    sgq  = dsigmagq_nlo(y,zz)/(1. - y/zz);

    sqg  = dsigmaqg_nlo(y,zz)/(1. - y/zz);
    
    sqq  = 0.;
    
    res  = pow(asr,3)*jac*s0*zz*(lgg*sgg + lqq*sqq
				 + lqg*sqg + lgq*sgq);// /tau;
  }
  return res;
}


double Massless_Limit::dsigmaqqb_nlo(double y, double z)
{
  return 0.;
}

double Massless_Limit::dsigmagg_nlo(double y, double z, double t)
{
  double res(0.);
  res = 2.*bb(y,z,t)+ 4.*bg(y,z)/(1. - y/z);
  return res;
}

double Massless_Limit::bb(double y, double z, double t){
  return bb_LO(y,z)/(1.-y/z) + bb_NLO(y,z,t);
}

double Massless_Limit::bb_LO(double y, double z)
{
  double res(0.);
  double L(log(m2h/m2b)), agb, pqg;
  pqg = L*Massless_Limit::p_lumi->Pqg(y/z)/(2.*M_PI);
  agb = (L*L*Massless_Limit::p_lumi->agb(z,2,2)
         + L*Massless_Limit::p_lumi->agb(z,2,1)
         + Massless_Limit::p_lumi->agb(z,2,0))/pow(2.*M_PI,2);

  res = 2.*y*pqg*agb/z;

  return res;
}

double Massless_Limit::bb_NLO(double y, double z, double t)
{
  double res(0.);
  double sc(0.);
  double L(log(m2h/m2b)), pqg1, pqg2, pqg20;
  double k=y/z;
  double sbb_d, sbb_p, sbb_sub, sbb_rest;

  sbb_d = delta1_(&sc)/M_PI; sbb_p = dterms1_(&t,&sc)/M_PI;
  sbb_sub = - dtsub1_(&sc,&k)/M_PI; sbb_rest = sbbq1_(&t,&sc)/M_PI;

  pqg1  = Massless_Limit::p_lumi->Pqg(z)/(2.*M_PI)*L;
  pqg20 = Massless_Limit::p_lumi->Pqg(y/z)/(2.*M_PI)*L;

  pqg2  = Massless_Limit::p_lumi->Pqg(y/z/t)/(2.*M_PI)*L;

  // delta bit:
  res = sbb_d*pqg20/(1. - y/z);

  // plus bit:
  double x = y/z;
  double fac =L*(4*pow(x,-1)*(2*(2 + 3*x*(-2 + 5*x))*log(1 - x) + 6*(1 - 3*x)*x*log(x) + 
			      6*x*(1 + 2*(-1 + x)*x)*dilog(x) + 
			      x*(4*(-1 + x)*(-1 + 3*x) + (-1 - 2*(-1 + x)*x)*pow(M_PI,2)) - 
			      44*atanh(1 - 2*x)*pow(x,3)))/9./M_PI/2./M_PI;    
  res += fac/(1.-y/z);

  // plus rest:
  res -= sbb_sub*pqg20/(1.-y/z);

  // rest:
  res += pqg2*sbb_rest/t/t;

  res*=y*pqg1/z;

  return res;
}

double Massless_Limit::bg(double y, double z)
{
  return bg_LO(y,z) + bg_NLO(y,z);
}

double Massless_Limit::bg_LO(double y, double z)
{
  double res(0.);
  double L(log(m2h/m2b)), agb;
  double sbg, sc(0.);

  // double k=y/z;

  agb = (Massless_Limit::p_lumi->agb(y/z,2,2)*L*L
         + Massless_Limit::p_lumi->agb(y/z,2,1)*L
         + Massless_Limit::p_lumi->agb(y/z,2,0))/pow(2.*M_PI,2);

  sbg = sbg1_(&z,&sc)/M_PI;

  res = y*sbg*agb/z/z;

  return res;
}

double Massless_Limit::bg_NLO(double y, double z)
{
  double res(0.);
  double L(log(m2h/m2b)), pqg;
  double sbg, sc(0.), nf(5);
  // double k=y/z;

  pqg = Massless_Limit::p_lumi->Pqg(y/z)*L/(2.*M_PI);
  sbg = (deltabga_(&z,&sc,&sc) +
         nf*deltabgf_(&z,&sc,&sc))/pow(M_PI,2);

  res = y*pqg*sbg/z/z;

  return res;
}

double Massless_Limit::dsigmagq_nlo(double y, double z)
{
  double res(0.);
  res = bbs(y,z) + 2.*bgs(y,z) + 2.*bqs(y,z);
  return res;
}

double Massless_Limit::dsigmaqg_nlo(double y, double z)
{
  return Massless_Limit::dsigmagq_nlo(y,z);
}

double Massless_Limit::bbs(double y, double z)
{
  double res(0.);
  double L(log(m2h/m2b)), asb, pqg;
  pqg = Massless_Limit::p_lumi->Pqg(y/z)/(2.*M_PI)*L;
  asb = (Massless_Limit::p_lumi->asb(z,2,2)*L*L
         + Massless_Limit::p_lumi->asb(z,2,1)*L
         + Massless_Limit::p_lumi->asb(z,2,0))/pow(2.*M_PI,2);

  res = 2.*y*pqg*asb/z;

  return res;
}

double Massless_Limit::bgs(double y, double z)
{
  double res(0.);
  double L(log(m2h/m2b)), asb;
  double sbg, sc(0.);
  // double k=y/z;
  asb = (Massless_Limit::p_lumi->asb(y/z,2,2)*L*L
         + Massless_Limit::p_lumi->asb(y/z,2,1)*L
         + Massless_Limit::p_lumi->asb(y/z,2,0))/pow(2.*M_PI,2);
  sbg = sbg1_(&z,&sc)/M_PI;

  res = y*sbg*asb/z/z;

  return res;
}

double Massless_Limit::bqs(double y, double z)
{
  double res(0.);
  double L(log(m2h/m2b)), pqg;
  double sbq, sc(0.), nf(5);
  // double k=y/z;
  pqg = Massless_Limit::p_lumi->Pqg(y/z)/(2.*M_PI)*L;
  sbq = (deltaqqba_(&z,&sc,&sc) +
         nf*deltaqqbf_(&z,&sc,&sc))/pow(M_PI,2);

  res = y*pqg*sbq/z/z;

  return res;

}

double Massless_Limit::muFct(double z[], size_t dim, void *p)
{
  double res(0.);
  // first round of variables
  double tau,xi,ximax,jac(1.),x,y;
  // lumis and MEs
  double lgg,lqg,lgq,lqq,
    sgg,sgq,sqg, sqq;
  //second variables
  double zz, tt;

  tau  = tauh + (1. - tauh)*z[0];
  jac *= (1. - tauh);

  ximax = -1./2.*log(tau);
  xi    = -ximax + 2.*ximax*z[1];
  jac  *= 2.*ximax;

  x = sqrt(tau)*exp(xi);
  y = sqrt(tau)*exp(-xi);

  lgg = Massless_Limit::p_lumi->Lumi(5,5,x,tauh/tau,1.,1.,10);
  lgq = Massless_Limit::L_gq(x,tauh/tau);
  lqg = Massless_Limit::L_gq(tauh/tau,x);
  lqq = Massless_Limit::L_qq(x,tauh/tau);

  zz = y + (1. - y)*z[2];
  jac *= (1. - y);
  tt = z[3];

  sgg = dsigmagg_st(y,zz,tt);
  sqg = dsigmagq_st(y,zz,tt);
  sgq = sqg;
  sqq = dsigmaqq_st(y,zz);
  // cout << jac << " , " << sgg << " , " << sqq << " , " << sqg << " , " <<sgq << endl;
  res  = jac*
    (lgg*sgg + lqq*sqq + lqg*sqg + lgq*sgq)/tau;

  return res;
}


double Massless_Limit::slogg(double y, double z, double r)
{
  double res(0.);
  double t = y/z + (1. - y/z)*r;
  double jac = (1. - y/z);

  res = jac*Massless_Limit::dsigmagg_lo(y/z,y/z/t)/t/t;
  return res;
}



double Massless_Limit::sloqq(double y, double z)
{
  double res(0.);
  res = (1. - y/z)*Massless_Limit::dsigmaqqb_lo(y/z,1.);
  return res;
}


/// Sub term
double Massless_Limit::dsigmagg_st(double y,double z, double kp)
{


  double ds(0.);

  double t = (1.-y/z)*kp + y/z,asr(p_lumi->ass(m2f)),
    k = (1.-y)*kp + y;
  double j = (1.-y/z);

  double ppc(p_lumi->Pgg_pt(z)),ppc0(p_lumi->Pgg_pt(1.));
  double pgg(p_lumi->Pgg(z)), d(p_lumi->Pgg_d());


  ds = 2.*j*
    ( pgg*dsigmagg_lo(y/z,t) + (d + ppc0*log(1.-y))*dsigmagg_lo(y,z)/(1.-y/z) +
      (ppc*dsigmagg_lo(y/z,t) - (1.-y)*(ppc0*dsigmagg_lo(y,k))/(1.-y/z))/(1.-z) );

  return ds;
}

double Massless_Limit::dsigmaqq_st(double y, double z)
{
  double ds(0.), asr(p_lumi->ass(m2f));

  double h1(p_lumi->Pqq_pt(1.)), h(p_lumi->Pqq_pt(z)), sqq(dsigmaqqb_lo(y/z,1.)),
    sqq1(dsigmaqqb_lo(y,1.)), d(p_lumi->Pqq_d());

  ds = 2.*((d + h1*log(1.-y))*sqq1/(1.-y) +
           (h*sqq - h1*sqq1)/(1.-z) );

  return ds;
}

double Massless_Limit::dsigmagq_st(double y,double z, double kp)
{
  double t = (1.-y/z)*kp + y/z;

  double ds(0.),asr(p_lumi->ass(m2f)),
    dsgg(dsigmagg_lo(y/z,t)), dsqq(dsigmaqqb_lo(y/z,1.)),
    nf = 5.;

  ds = (1.-y/z)*(p_lumi->Pgq(z)*dsgg
                 +p_lumi->Pqg(z)*dsqq);

  return ds;
}


double Massless_Limit::fun(double y, double z){
  double res = pow(y - 1.*z,-1)*pow(z,-4)*
    (y*dilog(1. - 1.*y*pow(z,-1))*pow(z,2)*
     (1.333*y*z - 2.67*pow(y,2) + 1.333*pow(z,2)) +
     y*(11.85*z*pow(y,3) - 0.1852*pow(y,4) -
	27.9*pow(y,2)*pow(z,2) + 22.6*y*pow(z,3) +
	log(y*pow(z,-1))*
	((0.667*y - 0.667*z)*z*log(y*pow(z,-1))*
	 pow(y,2) + 5.*z*pow(y,3) -
	 0.889*pow(y,4) -
	 13.44*pow(y,2)*pow(z,2) +
	 12.*y*pow(z,3) - 2.67*pow(z,4)) -
	6.32*pow(z,4)) +
     log(1. - 1.*y*pow(z,-1))*
     (-10.89*z*pow(y,4) + 1.778*pow(y,5) +
      23.8*pow(y,3)*pow(z,2) -
      23.8*pow(y,2)*pow(z,3) +
      y*z*log(y*pow(z,-1))*
      (2.67*z*pow(y,2) - 2.67*pow(y,3) -
       2.67*y*pow(z,2) + 2.67*pow(z,3)) +
      10.89*y*pow(z,4) - 1.778*pow(z,5)) +
     y*z*(-5.33*z*pow(y,2) + 2.67*pow(y,3) +
	  4.*y*pow(z,2) - 1.333*pow(z,3))*
     pow(log(1. - 1.*y*pow(z,-1)),2));
  return res;
}
