////
// Calculate the massless scheme at LO, NLO and NNLO
///

#include "Massless_lim.H"
#include "Massless_Kernels.H"

#include <iostream>
#include <vector>
#include <cmath>

using namespace std;

double Massless_Limit::dsigmaLO(double x[], size_t dim, void* p)
{
  double dsg(0.);
  double tau, jac(1.);
  tau  = tauh + (1. - tauh)*x[0];
  jac *= (1. - tauh);
   
  dsg = 2.*jac*p_lumi->Lumi(10,0,tau,tauh/tau,1.,1.,10)/tau;
  return s0*dsg;
}

double Massless_Limit::dsigmaNLO(double z[], size_t dim, 
				 void* p, const size_t plus) 
// here plus means should I do the subtraction? yes=1,no=0
{
  double as = 2.*p_lumi->ass(m2r);
  double dsg(0.);
  // delta contr
  if(dim==1){
    double tau, jac(1.);
    double sbb(0.);
    tau  = tauh + (1. - tauh)*z[0];
    jac *= (1. - tauh);
    sbb    = delta1_(&lfr);
    dsg = 2.*as*sbb*jac*p_lumi->Lumi(10,0,tau,tauh/tau,1.,1.,10)/tau;
    return s0*dsg;
  }
  // main thing
  if(dim==2 && plus == 0){
    double tau, jac(1.), xi, x, y;
    double sbbp(0.), sbb(0.), sbg(0.);
    tau  = tauh + (1. - tauh)*z[0];
    jac *= (1. - tauh);
    double ximax = - 0.5*log(tau);
    xi   = -ximax + 2.*ximax*z[1];
    jac *=  2.*ximax;

    // standard variables  x, y ;
    x = sqrt(tau)*exp(xi);
    y = sqrt(tau)*exp(-xi);

    sbbp    = dterms1_(&y, &lfh);
    sbb     = sbbq1_(&y, &lfh);
    sbg     = sbg1_(&y,&lfh);

    dsg     = 
      as*jac*(
	      2.*p_lumi->Lumi(10,0,x,tauh/tau,1.,1.,10)*
	      (sbbp + sbb) 
	      - 2.*p_lumi->Lumi(10,0,x,tauh/x,1.,1.,10)*( y*sbbp )
	      + (p_lumi->Lumi(10,5,x,tauh/tau,1.,1.,10) + 
		 p_lumi->Lumi(5,10,x,tauh/tau,1.,1.,10) +
		 p_lumi->Lumi(0,5,x,tauh/tau,1.,1.,10) + 
		 p_lumi->Lumi(5,0,x,tauh/tau,1.,1.,10))*( sbg )
	      )/tau;
    return s0*dsg;
  }

  if(dim==2 && plus == 1){
    double tau, jac(1.), xi, x, y;
    double sbbp(0.);
    tau  = tauh + (1. - tauh)*z[0];
    jac *= (1. - tauh);
    xi   = (tauh/tau)*z[1];
    jac *= (tauh/tau);
    x = tau; y = xi;
    sbbp    = dterms1_(&y, &lfh);

    dsg = 2.*jac*as*sbbp*p_lumi->Lumi(10,0,x,tauh/x,1.,1.,10)/x;
    return s0*dsg;
  }
  else if (dim > 2){
    cout << " No more than 2 dim for dsigmaNLO! " << std::endl;
    return 0.;
  }
  return s0*dsg;
}

double Massless_Limit::dsigmaNNLO(double z[], size_t dim, 
				  void* p, const size_t plus) 
// here plus means should I do the subtraction? yes=1,no=0
{
  //cout << " MODE:: "<< m_mode << std::endl;
  double as  = 2.*p_lumi->ass(m2r);
  double a2s = pow(as,2);
  double dsg(0.), nf(5.);
  // delta contr
  if(dim==1){
    double tau, jac(1.); double y(1.);
    double sbba(0.), sbbf(0.);
    tau  = tauh + (1. - tauh)*z[0];
    jac *= (1. - tauh);
    sbba    = delta2_(&lfr, &lfh);
    dsg = 2.*a2s*(sbba )*
      jac*p_lumi->Lumi(10,0,tau,tauh/tau,1.,1.,10)/tau;
    return s0*dsg;
  }
  // main thing
  if(dim==2){ 
    if (plus == 0) {
      double tau, jac(1.), xi, x, y;
      double lbb, lbb0, lbg, lbq, lgg,lqq, lbs;
      double sbbp(0.), sbba(0.), sbbf(0.),
	sbg(0.), sbq(0.), sgg(0.), sbs(0.), sqq(0.);
      
      tau  = tauh + (1. - tauh)*z[0];
      jac *= (1. - tauh);
      double ximax = - 0.5*log(tau);
      xi   = -ximax + 2.*ximax*z[1];
      jac *=  2.*ximax;

      // standard variables  x, y ;
      x = sqrt(tau)*exp(xi);
      y = sqrt(tau)*exp(-xi);

      // Deltaterms

      //soft-terms
      sbbp      =       dterms2_(&y,&lfh,&lfr);
      //      finite-terms
      //b b~
      sbba       =       deltabbqa_(&y,&lfh,&lfr);
      sbbf       =       deltabbqf_(&y,&lfh,&lfr);
      
      // b g
      sbg        =       deltabga_(&y,&lfh,&lfr) +
	nf* deltabgf_(&y,&lfh,&lfr);
      
      // g g
      sgg        =       deltagga_(&y,&lfh,&lfr);
      
      // b b
      sbs        =       deltabba_(&y,&lfh,&lfr);

      // b q      
      sbq        =       deltabqa_(&y,&lfh,&lfr);

      // q q~
      sqq        =       deltaqqba_(&y,&lfh,&lfr);

      // luminosities 
      lbb     = 2.*p_lumi->Lumi(10,0,x,tauh/tau,1.,1.,10);
    
      lbb0    = 2.*y*p_lumi->Lumi(10,0,x,tauh/x,1.,1.,10);

      lbg     = p_lumi->Lumi(10,5,x,tauh/tau,1.,1.,10) + 
	p_lumi->Lumi(5,10,x,tauh/tau,1.,1.,10) +
	p_lumi->Lumi(0,5,x,tauh/tau,1.,1.,10) + 
	p_lumi->Lumi(5,0,x,tauh/tau,1.,1.,10);

      lbq     = 0.;
   
      lbq += 2.*( p_lumi->Lumi(10,1,x,tauh/tau,1.,1.,10)       // d
		  + p_lumi->Lumi(10,6,x,tauh/tau,1.,1.,10));
      lbq += 2.*( p_lumi->Lumi(1,10,x,tauh/tau,1.,1.,10)       
		  + p_lumi->Lumi(6,10,x,tauh/tau,1.,1.,10));
      lbq += 2.*( p_lumi->Lumi(10,2,x,tauh/tau,1.,1.,10)       // u
		  + p_lumi->Lumi(10,7,x,tauh/tau,1.,1.,10));
      lbq += 2.*( p_lumi->Lumi(2,10,x,tauh/tau,1.,1.,10) 
		  + p_lumi->Lumi(7,10,x,tauh/tau,1.,1.,10));
      lbq += 2.*( p_lumi->Lumi(10,3,x,tauh/tau,1.,1.,10)       // s
		  + p_lumi->Lumi(10,8,x,tauh/tau,1.,1.,10));
      lbq += 2.*( p_lumi->Lumi(3,10,x,tauh/tau,1.,1.,10) 
		  + p_lumi->Lumi(8,10,x,tauh/tau,1.,1.,10));

      lbq += 4.*( p_lumi->Lumi(10,4,x,tauh/tau,1.,1.,10) );  // (c = c~)
      lbq += 4.*( p_lumi->Lumi(4,10,x,tauh/tau,1.,1.,10) );
     

      lgg     = p_lumi->Lumi(5,5,x,tauh/tau,1.,1.,10);

      lqq     = 0.;
      
      lqq +=  p_lumi->Lumi(4,6,x,tauh/tau,1.,1.,10);
      lqq +=  p_lumi->Lumi(6,4,x,tauh/tau,1.,1.,10);
      lqq +=  p_lumi->Lumi(3,7,x,tauh/tau,1.,1.,10);
      lqq +=  p_lumi->Lumi(7,3,x,tauh/tau,1.,1.,10);
      lqq +=  p_lumi->Lumi(2,8,x,tauh/tau,1.,1.,10);
      lqq +=  p_lumi->Lumi(8,2,x,tauh/tau,1.,1.,10);
      lqq +=  p_lumi->Lumi(1,9,x,tauh/tau,1.,1.,10);
      lqq +=  p_lumi->Lumi(9,1,x,tauh/tau,1.,1.,10);
      
      lbs     = 2.*p_lumi->Lumi(10,10,x,tauh/tau,1.,1.,10); // lhaf + 5 - last entry, if 10 it takes PDF from file
                                                            //                        if 1 btilde as*L
                                                            //                        if 2 btilde (as*L)^2 + as^2 * L
      //if (m_mode <= 0 ){
	dsg     = 
	  a2s*jac*( lbb * ( sbbp + sbba + nf * sbbf)
		    - lbb0 * sbbp
		    + lbg  * sbg
		    + lgg  * sgg
		    + lbs  * sbs
		    + lbq  * sbq
		    + lqq  * sqq
		    )/tau; 
	//}
      // if (m_mode > 0){
      // 	dsg     = 
      // 	  a2s*jac*( lbb * ( sbbp + sbba + nf * sbbf)
      // 		    - lbb0 * sbbp
      // 		    + lbg  * sbg
      // 		    + lbs  * sbs
      // 		    + lbq  * sbq
      // 		    )/tau; 
      // }
      return s0*dsg;
    }
    
    if(plus == 1){
      double tau, jac(1.), xi, x, y;
      double sbbp(0.);
      tau  = tauh + (1. - tauh)*z[0];
      jac *= (1. - tauh);
      xi   = (tauh/tau)*z[1];
      jac *= (tauh/tau);
      x = tau; y = xi;
      sbbp    = dterms2_(&y,&lfh,&lfr);

      dsg = 2.*jac*a2s*sbbp*p_lumi->Lumi(10,0,x,tauh/x,1.,1.,10)/x;
      return s0*dsg;
    }
  }
  else if (dim > 2){
    cout << " No more than 2 dim for dsigmaNLO! " << std::endl;
    return 0.;
  } 
  return s0*dsg;
}


